'use strict'

var async = require('async')

var content = require('./content.js')
var dwf = require('./dwf.js')
var logger = require('./logger.js')
var filters = require('./filters.js')

module.exports = function (options, services, cb) {
  var contentPathsFilter = filters.invalidPathFilter()
  if (options.includeFilter.valid) {
    // an include-filter is given, merge the include-filter and the deploy options to ensure that we upload all files which are included in the deploy option
    // if no include-filter is given, we don't need to care about the deploy option during the file walk
    contentPathsFilter.addPaths(options.includeFilter)
    contentPathsFilter.addPaths(options.deploy)
  }

  var cnt = content(options.root, services, options.deployDirs, contentPathsFilter, options.stripCRFromCSV)
  var tasks = []
  var targetSrvMin

  logger.trace('VCAP_SERVICES:', process.env.VCAP_SERVICES)

  logger.trace('top directories: ', cnt.serverTopDirs())

  logger.trace('deploy files: ', cnt.deployFiles())

  targetSrvMin = services.getTargetSrvMin()
  logger.trace('target service min:', targetSrvMin)

  tasks.push(function (cb) {
    dwf.deploy(options, targetSrvMin, cnt, cb)
  })

  async.series(tasks, cb)
}
