'use strict'
const Result = require('../../result.js').Result

const coAsyncLib = require('async')
const coAbstractLib = require('./abstract.js')
const coValidateLib = require('../../validate.js')
const coDwfAbstract = require('../../abstract.js')
const coRepositoryLib = require('../../repository.js')

const cfPostBatch = function (isExt, iaoTaskChainPostBatch, iaoResultItem, ioProviderExt, ioCredential) {
  let laoObjectInstanceToDelete = []
  let laoObjectInstanceToPost = []
  let laoResultItem = iaoResultItem

  iaoTaskChainPostBatch.forEach(function (lObj) {
    if (lObj[coAbstractLib.csKey_AOI_Action] === coAbstractLib.Methods.POST) {
      laoObjectInstanceToPost.push(lObj.data)
    }
  })
  laoResultItem.forEach(function (ioResultItem) {
    if (ioResultItem[coAbstractLib.csKey_AOI_Action] === coAbstractLib.Methods.DELETE) {
      let loObjectInstance = {
        'namespace': ioResultItem[coAbstractLib.csKey_AOI_csNamespace],
        'taskChainId': ioResultItem[coAbstractLib.csKey_AOI_Task_chain_Id],
        'clear': true
      }
      laoObjectInstanceToDelete.push(loObjectInstance)
    }
  })

  let lafPost = coAbstractLib.fpreparePost(isExt, laoObjectInstanceToPost, ioProviderExt, ioCredential, coAbstractLib.Methods.POST)
  let lafDelete = coAbstractLib.fpreparePost(isExt, laoObjectInstanceToDelete, ioProviderExt, ioCredential, coAbstractLib.Methods.DELETE)
  let lafTask = []
  if (lafPost.length > 0) {
    lafTask = lafTask.concat(lafPost)
  }
  if (lafDelete.length > 0) {
    lafTask = lafTask.concat(lafDelete)
  }
  return lafTask
}

const cfPostRepo = function (isExt, iaoResultItem, ioProviderExt, ioObjectProperties, ioCredential, ioOption, ioProviders, ioObjectType) {
  let loRepository = new coRepositoryLib.Repository(ioCredential, ioOption)
  let loObjectProperties = ioObjectProperties
  let lafTaskWaterfall = loRepository.get_afTask2Update(isExt, iaoResultItem, ioProviders, ioObjectType, loObjectProperties)
  let lafTask = []
  lafTask.push(function (ifCb) {
    coAsyncLib.waterfall(lafTaskWaterfall, function (ioError, ioResult) {
      ifCb(ioError, ioResult)
    })
  })
  return lafTask
}

const coType = {}
coType[coAbstractLib.csExtention] = '.dwftaskchain'
coType[coAbstractLib.cfPrepare] = undefined
coType[coAbstractLib.cfIsDeployable] = function () {
  let fCreate = coType[coAbstractLib.cfCreateTasks]
  if (fCreate) {
    return true
  } else {
    return false
  }
}
coType[coAbstractLib.cfCommonObjectProperties] = function () {
  return {
    'NAMESPACE': coAbstractLib.csKey_AOI_csNamespace,
    'ID': coAbstractLib.csKey_AOI_Task_chain_Id,
    'TYPE_ID': coAbstractLib.csKey_AOI_TYPE_ID,
    'DESIGN_TIME_FILE_PATH': coAbstractLib.csKey_AOI_sFile,
    'HASH_UTF8_SHA256_HEX': coAbstractLib.csKey_AOI_sHashUtf8Sha256Hex
  }
}
coType[coAbstractLib.cfCreateTasks] = function (iaoPostBatch, ioResult, ioCredential, ioOption, ioProviders, ioObjectType) {
  coValidateLib(iaoPostBatch.length, 'number')
  coValidateLib(ioResult, 'object')
  coValidateLib(ioCredential, 'object')
  coValidateLib(ioOption, 'object')
  let loTempResultObj = new Result()
  let loResult = ioResult
  let lsExt = coType[coAbstractLib.csExtention]
  let laoPostBatch = iaoPostBatch
  let loValue4afTask = []

  let loProviderExt = coAbstractLib.cfGetProvider(ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1], lsExt)
  let lafTaskPostBatch = cfPostBatch(lsExt, laoPostBatch, loResult, loProviderExt, ioCredential)
  loValue4afTask = loValue4afTask.concat(lafTaskPostBatch)

  let loObjectProperties = coType[coAbstractLib.cfCommonObjectProperties]()
  let lafTaskPostRepo = cfPostRepo(lsExt, loResult, loProviderExt, loObjectProperties, ioCredential, ioOption, ioProviders, ioObjectType)
  loValue4afTask = loValue4afTask.concat(lafTaskPostRepo)

  loTempResultObj.addValue(lsExt, loValue4afTask)

  return loTempResultObj
}

module.exports = coType
