'use strict'

let Ajv = require('ajv')

var ajv = new Ajv({
  verbose: true,
  allErrors: true,
  extendRefs: true,
  schemaId: 'auto'
})
ajv.addMetaSchema(require('ajv/lib/refs/json-schema-draft-04.json'))
ajv.addMetaSchema(require('ajv/lib/refs/json-schema-draft-06.json'))

ajv.addSchema(require('../object/schema/dwfnamespace.schema.json'), '.dwfnamespace')
function getSeverity () {
  return {
    warning: 'warning',
    error: 'error',
    info: 'info'
  }
}
exports.validate = function (schema, obj) {
  let severity = getSeverity()
  let retval = ajv.validate(schema, obj)
  let tempMsg = ''
  if (ajv && ajv.errors) {
    ajv.errors.forEach(function (err) {
      tempMsg = tempMsg + err.message + ' '
      if (err.params) {
        tempMsg = tempMsg + JSON.stringify(err.params) + '  ' + JSON.stringify(err.schemaPath) + ' '
      }
    })
  }
  let msg = 'Schema Validation success'
  if (!retval) {
    msg = ajv.errors ? { msg: tempMsg, severity: severity.error } : { msg: 'Schema Validation failed' }
  }
  return {
    valid: retval,
    errors: ajv.errors,
    msg: msg
  }
}
