'use strict'

const coLoggerLib = require('../logger.js')
const coRequestLib = require('./request.js')
const coValidateLib = require('./validate.js')
const coDwfAbstract = require('./abstract.js')
const coAbstractLib = require('./object/type/abstract.js')

function Repository (ioCredential, ioOption) {
  this.loCredential = coValidateLib(ioCredential, 'object')
  this.loOption = coValidateLib(ioOption, 'object')
}

Repository.prototype.get_afTask2Get = function (URL) {
  let lafTask // note - waterfall stile tasks
  let loRepository = this.loCredential[coDwfAbstract.cs_Credential_Key_Repository_v1]
  let loData = {
    uri: undefined, // note - update me on each step
    json: true,
    method: undefined, // note - update me on each step
    auth: {
      user: this.loCredential.user,
      pass: this.loCredential.password,
      sendImmediately: true
    },
    body: undefined // note - update me on each step
  }

  // Object Types - GET
  lafTask = function (ifCb) {
    loData.uri = loRepository.url + '/' + URL
    loData.method = coAbstractLib.Methods.GET
    loData.body = undefined

    coLoggerLib.trace(' * Repository - ' + loData.uri + ' ' + loData.method + ' ... ')
    let lfCb = coAbstractLib.lfCb2Reuse('Repository', loData, ifCb)
    coRequestLib(loData, lfCb)
  }
  return lafTask
}

Repository.prototype.get_afTask2Update = function (isExt, iaoResultItem, ioServiceProvider, ioObjectType, ioObjectProperties) {
  let lafTask = [] // note - waterfall stile tasks

  let laoObjectInstanceToUpdate = []
  let laoObjectInstanceToDelete = []
  let loObjectProperties = ioObjectProperties// change it and make a parameter (coAbstractLib.coExtentions.hasOwnProperty(isExt)) ? coAbstractLib.coExtentions[isExt] : {}
  iaoResultItem.forEach(function (ioResultItem) {
    let loObjectInstance = {}
    Object.keys(loObjectProperties).forEach(function (key) {
      loObjectInstance[key] = ioResultItem[loObjectProperties[key]]
    })
    if (ioResultItem.sAction === coAbstractLib.Methods.POST) {
      laoObjectInstanceToUpdate.push(loObjectInstance)
    } else if (ioResultItem.sAction === coAbstractLib.Methods.DELETE) {
      laoObjectInstanceToDelete.push(loObjectInstance)
    }
  })

  let loRepository = this.loCredential[coDwfAbstract.cs_Credential_Key_Repository_v1]
  let loData = {
    uri: undefined, // note - update me on each step
    json: true,
    method: undefined, // note - update me on each step
    auth: {
      user: this.loCredential.user,
      pass: this.loCredential.password,
      sendImmediately: true
    },
    body: undefined // note - update me on each step
  }

  // Service Providers - POST - no checks are required, because of the upsert in the backend
  lafTask.push(function (ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.PROVIDERS
    loData.method = coAbstractLib.Methods.POST
    loData.body = ioServiceProvider[coAbstractLib.Methods.POST]

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) {
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb(null, null)
    }
  })

  // Service Providers - DELETE - no checks are required, because of the upsert in the backend
  lafTask.push(function (iaoProviderPost, ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.PROVIDERS
    loData.method = coAbstractLib.Methods.DELETE
    loData.body = ioServiceProvider[coAbstractLib.Methods.DELETE]

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) {
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb(null, null)
    }
  })

  // Object Types - POST
  lafTask.push(function (iaoProviderPost, ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.OBJECT_TYPES
    loData.method = coAbstractLib.Methods.POST
    loData.body = ioObjectType[coAbstractLib.Methods.POST]

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) {
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb(null, null)
    }
  })
  // Object Types - POST
  lafTask.push(function (iaoProviderPost, ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.OBJECT_TYPES
    loData.method = coAbstractLib.Methods.DELETE
    loData.body = ioObjectType[coAbstractLib.Methods.DELETE]

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) {
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb(null, null)
    }
  })

  // Object - DELETE
  lafTask.push(function (iaoProviderPost, ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.OBJECTS
    loData.method = coAbstractLib.Methods.DELETE
    loData.body = laoObjectInstanceToDelete

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) { // Check - do not post with empty array
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb(null, null)
    }
  })

  // Object - POST
  lafTask.push(function (iaoProviderPost, ifCb) {
    loData.uri = loRepository.url + '/' + coAbstractLib.TYPES.OBJECTS
    loData.method = coAbstractLib.Methods.POST
    loData.body = laoObjectInstanceToUpdate

    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ')
    if (loData.body.length > 0) { // Check - do not post with empty array
      let lfCb = coAbstractLib.lfCb2Reuse(isExt, loData, ifCb)
      coRequestLib(loData, lfCb)
    } else {
      coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... skipped' + '(body = ' + JSON.stringify(loData.body) + ')')
      ifCb()
    }
  })

  return lafTask
}

module.exports = {
  Repository
}
