'use strict'

const coValidateLib = require('./validate.js')
const coAbstractLib = require('./object/type/abstract.js')

function Result () {
  coValidateLib(arguments.length === 0, 'boolean')

  this[coAbstractLib.csKey_nStatus] = coValidateLib(coAbstractLib.cnSuccess, 'number')
  this[coAbstractLib.csKey_sDescription] = ''
  this[coAbstractLib.csKey_o_sExt2aoItem] = {}
  this[coAbstractLib.csKey_o_sExt2oValue] = {}
}

Result.prototype.addItem = function (isExt, ioItem) {
  coValidateLib(isExt, 'string')
  coValidateLib(ioItem, 'object')

  let lnStatus = coValidateLib(ioItem[coAbstractLib.csKey_AOI_nStatus], 'number')
  coValidateLib(ioItem[coAbstractLib.csKey_AOI_sFile], 'string')
  coValidateLib(ioItem[coAbstractLib.csKey_AOI_sDescription], 'string')

  if (this[coAbstractLib.csKey_nStatus] < lnStatus) {
    this[coAbstractLib.csKey_nStatus] = lnStatus
  }
  let laoItem = this[coAbstractLib.csKey_o_sExt2aoItem][isExt]
  if (typeof laoItem === 'undefined') {
    laoItem = []
  }
  laoItem.push(ioItem)
  this[coAbstractLib.csKey_o_sExt2aoItem][isExt] = laoItem
}

Result.prototype.addValue = function (isExt, ioValue) {
  coValidateLib(isExt, 'string')
  coValidateLib(ioValue, 'object')

  let loValue = this[coAbstractLib.csKey_o_sExt2oValue][isExt]
  if (typeof loValue !== 'undefined') {
    throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
      JSON.stringify(loValue) + ' and cannot be mapped again to ' + JSON.stringify(ioValue))
  }
  this[coAbstractLib.csKey_o_sExt2oValue][isExt] = ioValue
}

Result.prototype.addResult = function (ioResult) {
  coValidateLib(ioResult, 'object')

  let that = this
  if (this[coAbstractLib.csKey_nStatus] < ioResult[coAbstractLib.csKey_nStatus]) {
    this[coAbstractLib.csKey_nStatus] = ioResult[coAbstractLib.csKey_nStatus]
    this[coAbstractLib.csKey_sDescription] = ioResult[coAbstractLib.csKey_sDescription]
  }
  Object.getOwnPropertyNames(ioResult[coAbstractLib.csKey_o_sExt2aoItem]).forEach(function (isExt) {
    let laoItemDelta = ioResult[coAbstractLib.csKey_o_sExt2aoItem][isExt]
    let laoItem = that[coAbstractLib.csKey_o_sExt2aoItem][isExt]
    if (typeof laoItem !== 'undefined') {
      throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
        JSON.stringify(laoItem) + ' and cannot be mapped again to ' + JSON.stringify(laoItemDelta))
    }
    that[coAbstractLib.csKey_o_sExt2aoItem][isExt] = laoItemDelta
  })
  Object.getOwnPropertyNames(ioResult[coAbstractLib.csKey_o_sExt2oValue]).forEach(function (isExt) {
    let loValueDelta = ioResult[coAbstractLib.csKey_o_sExt2oValue][isExt]
    let loValue = that[coAbstractLib.csKey_o_sExt2oValue][isExt]

    if (typeof loValue !== 'undefined') {
      throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
        JSON.stringify(loValue) + ' and cannot be mapped again to ' + JSON.stringify(loValueDelta))
    }
    that[coAbstractLib.csKey_o_sExt2oValue][isExt] = loValueDelta
  })
}

module.exports.Result = Result
