'use strict'

var fs = require('fs')
var lPath = require('path')
var paths = require('./paths.js')
var utils = require('./utils.js')

const validator = require('./dwf/object/validator.js')

let pathSeperator
let baseDir
let errorMap = new Map()
let errorStack = ''
// helper function for calculating nameSpaces and rteturning a map object
function getNameSpacesHelper (dir, ionameSpaceInfo, rfMap) {
  let localDirs = [] // For keeping record of directories in current directory
  let fileWithNameSpace // it is undefined as long as we don't find any .hdinamespace file in current directory
  let defualtNSObject = ionameSpaceInfo.nameSpace // keeping track of current namespace to carry into sub directories
  let lmNameSpaceWithDir = ionameSpaceInfo.map
  dir = paths.clientPath(dir)
  // dir has whole path i.e  src/subdir/subsubdir this code block is just to split whole path and last directory in the path. in simple words, to get name of current directory.
  let currentDir = dir.split(pathSeperator)
  if (currentDir) {
    let cdLength = currentDir.length - 1
    while (cdLength >= 0) {
      if (currentDir[cdLength] !== '') {
        currentDir = currentDir[cdLength]
        cdLength = 0
      }
      cdLength--
    }
  }
  // Reding current directory using absolute path
  var files = fs.readdirSync(dir)
  files.forEach(function (ioFile) {
    var name = dir + pathSeperator + ioFile
    if (fs.statSync(name).isFile()) {
      var sExtnameLowerCase = paths.extnameNamelessFile(name).toLowerCase()
      if (sExtnameLowerCase === '.dwfnamespace') {
        try {
          // if file is with extention hdinamespace it calls get JsonObject to get a json object out of file.
          fileWithNameSpace = utils.readJSONFile(name)
          let val = validator.validate(sExtnameLowerCase, fileWithNameSpace)
          if (val && val.valid) {
            // empty
          } else {
            let key = lPath.normalize(name)
            errorMap.set(key, val.msg)
            errorStack = errorStack + ' \n ' + key + ':0:0 error : ' + JSON.stringify(val.msg)
          }
        } catch (e) {
          throw (e) // throw or emit?
        }
      }
    } else if (fs.statSync(name).isDirectory()) {
      // adding directories in the array for recursive call later.
      localDirs.push(ioFile)
    } else {
      console.log('Neither a file nor a directory :o ')
    }
  })
  // instead of whole path c://.... we will keep only path from directory which is received as parameter
  let directoryPath
  if (dir === baseDir) {
    directoryPath = dir
  } else {
    directoryPath = dir.split(baseDir)
    if (directoryPath) {
      if (directoryPath.length === 1) {
        directoryPath = directoryPath[0]
      } else if (directoryPath.length > 1 && directoryPath[directoryPath.length - 1] !== '') {
        directoryPath = directoryPath[directoryPath.length - 1]
      }
      let lastIndex = directoryPath.lastIndexOf(pathSeperator)
      if (lastIndex === directoryPath.length - 1) {
        directoryPath = directoryPath.substring(0, lastIndex)
      }
    } else {
      // need to decide
    }
  }
  directoryPath = lPath.normalize(directoryPath) // removes multiple // or .. in the path
  directoryPath = paths.clientPath(directoryPath)

  // if we find a .hdinamespace file then we get name from file and add directory name and namespace in the map.
  if (fileWithNameSpace) {
    defualtNSObject = fileWithNameSpace
    rfMap[directoryPath] = defualtNSObject.name
    ionameSpaceInfo.nameSpace = defualtNSObject
    if (!lmNameSpaceWithDir.hasOwnProperty(defualtNSObject.name)) {
      lmNameSpaceWithDir[defualtNSObject.name] = directoryPath
      ionameSpaceInfo.map = lmNameSpaceWithDir
    } else {
      console.log('wrong update of directory please check')
    }
  } else { // if we don't find any .hdinamespace then we check if there is namespace object from previous dir ?
    if (defualtNSObject && defualtNSObject.subfolder) {
      // special case which comes in test cases only

      // if namespace object is there and subfolder value is append: then we use name/dirname and add it to mapobject and defualtNSObject to use it in future.
      if (defualtNSObject.subfolder.toLocaleLowerCase() === 'append') {
        if (!fileWithNameSpace && lmNameSpaceWithDir.hasOwnProperty(defualtNSObject.name)) {
          let lsLastNameSpaceDir = lmNameSpaceWithDir[defualtNSObject.name]
          if (lsLastNameSpaceDir === currentDir) {
            lsLastNameSpaceDir = lsLastNameSpaceDir + pathSeperator
          }
          let lsPath2Append = directoryPath.split(lsLastNameSpaceDir)
          lsPath2Append = (lsPath2Append.length > 1) ? lsPath2Append[1] : ''
          if (lsLastNameSpaceDir === '') {
            lsPath2Append = directoryPath
          }
          lsPath2Append = pathSeperator + lsPath2Append
          lsPath2Append = lsPath2Append.split('/').join('.')
          lsPath2Append = lsPath2Append.split('..').join('.')
          let lsFinalAddition = (lsPath2Append === '.') ? '' : lsPath2Append
          let lsNS = defualtNSObject.name + lsFinalAddition
          rfMap[directoryPath] = lsNS
        } else {
          let lsNS = defualtNSObject.name + '.' + currentDir
          rfMap[directoryPath] = lsNS
          ionameSpaceInfo.nameSpace.name = lsNS
          ionameSpaceInfo.nameSpace.subfolder = 'append'
        }
      } else if (defualtNSObject.subfolder === 'ignore') { // if namespace subfolder value is ignore then we use only name of namespace and don't add dirname (path is absolute path to current dir)
        rfMap[directoryPath] = defualtNSObject.name
      } else {
        throw new Error('Unsupported  "' + defualtNSObject.subfolder + '" subfolder value ')
      }
    }
  }

  // loop on  all the directories which were found in the current Directory (recursive call)
  localDirs.forEach(function (ioLocalDir) {
    getNameSpacesHelper(dir + pathSeperator + ioLocalDir, ionameSpaceInfo, rfMap)
  })
  // if(!fileWithNameSpace) {
  let lsParentDir = paths.dirname(directoryPath)
  if (rfMap.hasOwnProperty(lsParentDir)) {
    ionameSpaceInfo.nameSpace.name = rfMap[lsParentDir]
  } else {
    ionameSpaceInfo.nameSpace = ionameSpaceInfo.defaultNameSpace
  }
  //  }

  return rfMap
}

function setPathSeperator () {
  // need to implement logic for windows and linux
  pathSeperator = '/'
  pathSeperator = paths.clientPath(pathSeperator)
}

function getGlobalNameSpace () {
  let mtaName
  let moduleName
  if (process && process.env && process.env.MTA_METADATA) {
    mtaName = JSON.parse(process.env.MTA_METADATA).id
  } else {
    // what to  do ??
  }
  if (process && process.env && process.env.MTA_MODULE_METADATA) {
    moduleName = JSON.parse(process.env.MTA_MODULE_METADATA).name
  } else {
    // what to do ??
  }
  let nsObject = {}
  nsObject.name = mtaName + '.' + moduleName
  nsObject.subfolder = 'append'
  return nsObject
}

function getNameSpaces (dirs, nameSpace) {
  let loMap = {}
  setPathSeperator()
  if (!Array.isArray(dirs)) {
    console.log('There are no directories to check')
    return loMap
  }
  // if namespace object is undefined or null
  if (!nameSpace) {
    nameSpace = getGlobalNameSpace()
  }
  let finalDirs = []

  dirs.forEach(function (d) {
    d = paths.clientPath(d)
    if (fs.existsSync(d)) {
      finalDirs.push(d)
    }
  })
  // let lonameSpaceInfo = {'nameSpace': {}, 'map' : {}, 'defaultNameSpace'}
  let lonameSpaceInfo = { 'nameSpace': {}, 'map': {}, 'defaultNameSpace': {} }
  lonameSpaceInfo.nameSpace = nameSpace
  lonameSpaceInfo.defaultNameSpace = nameSpace
  let lmTemp = {}
  lmTemp[nameSpace.name] = ''
  lonameSpaceInfo.map = lmTemp
  finalDirs.forEach(function (dir) {
    dir = paths.clientPath(dir) // it converts '\' to '/' or vice versa depending on clientPath
    baseDir = paths.dirname(dir)
    baseDir = paths.dirname(baseDir) // getting parent directory .
    if (baseDir === '.') {
      baseDir = './'
    }
    loMap = getNameSpacesHelper(dir, lonameSpaceInfo, loMap)

    if (errorMap && errorMap.size > 0) {
      throw errorStack
    } else {
      // console.log(' namespace schema validation passed  ')
    }
  })

  return loMap
}
module.exports = {
  getNameSpaces: getNameSpaces,
  getGlobalNameSpace: getGlobalNameSpace
}
