'use strict'

var xsenv = require('@sap/xsenv')
var tagDwfRuntime = 'dwf'

function ServiceAccessor () {
  this.services = xsenv.filterCFServices(function (service) {
    if (service.tags && service.tags.indexOf(tagDwfRuntime) !== -1) {
      return true
    }
    if (service.label === 'user-provided' && service.credentials.tags && service.credentials.tags.indexOf(tagDwfRuntime) !== -1) {
      return true
    }
    return false
  })

  // returns the service which represents the target container
  this.getTarget = function () {
    if (process.env.TARGET_CONTAINER) {
      return this.getServiceOrThrow(process.env.TARGET_CONTAINER)
    }

    switch (this.services.length) {
      case 0:
        throw new Error('no service definition found there must be at least one service definition for the deployment target')
      case 1:
        return this.services[0]
      default:
        throw new Error('more than one service definition found, but no service is defined as the deployment target via the environment variable "TARGET_CONTAINER"')
    }
  }

  this.getTargetSrvMin = function () {
    var targetAll = this.getTarget() || {}
    var targetMin = {}
    targetMin.name = targetAll.name
    targetMin.credentials = targetAll.credentials
    return targetMin
  }
}

module.exports = function () {
  return new ServiceAccessor()
}
